import { Injectable } from '@angular/core';
import { Observable, Subject } from 'rxjs';
import { map, mergeMap } from 'rxjs/operators';
import { HttpClient } from '@angular/common/http';
import { Organization, IOrganization, OrganizationRO } from '../models/organization.model';
import { APP_CONFIG } from './app-config.service';

@Injectable()
export class OrganizationService {

  limit: number;
  pageNumber: number;
  sortOptions: {
    value: string,
    order: string,
  };

  private _searchChangeSubject: Subject<any>;

  constructor(
    private _httpClient: HttpClient) {
    this._searchChangeSubject = new Subject<any>();
    this.limit = 10;
    this.pageNumber = 1;
  }

  getOrganizations(options?): Observable<OrganizationRO> {
    let query = '?';
    query += `limit=${(this.limit ? this.limit : 20)}`;
    query += `&offset=${(this.pageNumber ? (this.pageNumber - 1) * this.limit : 0)}`;
    query += `&sort_by=${this.sortOptions.value}.${this.sortOptions.order}`;

    return this._httpClient.get<IOrganization[]>(APP_CONFIG.organizations.url + query, { observe: 'response' }).pipe(
      map((response) => {
        const totalCount = response.headers.get('Content-Range');
        const organizations = [];
        response.body.forEach((organization) => {
          organizations.push(new Organization(organization));
        });
        return new OrganizationRO(organizations, parseInt(totalCount, 10));
      }));
  }

  findById(id): Observable<Organization> {
    return this._httpClient.get<IOrganization>(APP_CONFIG.organizations.url + id).pipe(
      map((response) => {
        return new Organization(response);
      }),
    );
  }

  delete(id) {
    return this._httpClient.delete(APP_CONFIG.organizations.url + id);
  }

  uploadLogoAndSaveOrganization(logoFile, organization) {
    return this.uploadLogo(logoFile).pipe(
      mergeMap((response: any) => {
        organization.logo = response.mediaUrl;
        return this.replaceOrCreate(organization);
      }),
    )
  }

  uploadLogo(logoFile): Observable<string> {
    const formData = new FormData();
    formData.append('file', logoFile);

    return this._httpClient.post<string>(`${APP_CONFIG.mediaLibrary.url}media`, formData).pipe(
      map((response) => {
        return response;
      }),
    );
  }

  replaceOrCreate(data: Organization): Observable<Organization> {
    if (data.id) {
      return this._httpClient.put<IOrganization>(APP_CONFIG.organizations.url + data.id, data).pipe(
        map((response) => {
          return new Organization(response);
        }),
      );
    }

    return this._httpClient.post<IOrganization>(APP_CONFIG.organizations.url, data).pipe(
      map((response) => {
        return new Organization(response);
      }),
    );
  }

  /* PAGINATION */
  paginationChanged(limit: number, pageNumber: number) {
    this.limit = limit;
    this.pageNumber = pageNumber;
    this._searchChangeSubject.next();
  }

  reverseSortOrder(): void {
    if (this.sortOptions.order === 'asc') {
      this.sortOptions.order = 'desc';
    } else {
      this.sortOptions.order = 'asc';
    }
  }

  get searchChange$(): Observable<string> {
    return this._searchChangeSubject.asObservable();
  }
}
