import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, Subject } from 'rxjs';
import { IUserInfo, User, ILoginResponse } from '../models';
import { map, mergeMap, tap } from 'rxjs/operators';
import * as NodeRSA from 'node-rsa';
import { APP_CONFIG } from '../../services/app-config.service';
import { NotificationService } from '../../services';
import { Router } from '@angular/router';

@Injectable()
export class UserService {

  private _user: User = null;
  _userStatusChangedSubject: Subject<boolean>;

  constructor(
    private _http: HttpClient,
    private _notificationService: NotificationService,
  ) {
    this._userStatusChangedSubject = new Subject<boolean>();
  }

  initializeService() {
    this.setUserInfo();
  }

  // Function and helpers allowing the management of the user session (jwt), info...
  setSession(authResult): boolean {
    let success = false;
    if (authResult && authResult.userInfo && authResult.xsrfToken) {
      localStorage.setItem('userInfo', JSON.stringify(authResult.userInfo));
      localStorage.setItem('xsrfToken', authResult.xsrfToken);
      this.setUserInfo();
      success = true;
      this._userStatusChangedSubject.next(true);
    } else {
      this.resetAuth();
    }

    return success;
  }

  setUserInfo() {
    const userInfo = JSON.parse(localStorage.getItem('userInfo'));
    if (userInfo) {
      this._user = new User(userInfo);
    }
  }

  resetAuth() {
    localStorage.removeItem('userInfo');
    localStorage.removeItem('xsrfToken');
    this.logout().subscribe();
    this._user = null;
    this._userStatusChangedSubject.next(false);
  }

  get user() {
    return this._user;
  }

  get userIsSignedIn() {
    return this._user != null;
  }

  get userStatusChanged$(): Observable<boolean> {
    return this._userStatusChangedSubject.asObservable();
  }

  // HTTP Calls
  login(loginForm): Observable<boolean> {
    // Make sure not to mofidy the object passed in parameter (object reference) when encrypting the password
    const form = Object.assign({}, loginForm);
    return this.getPublicKey().pipe(
      tap(
        (publicKey) => {
          form.password = this.encrypt(form.password, publicKey);
        },
      ),
      mergeMap(() => {
        return this._http.post<ILoginResponse>(
          `${APP_CONFIG.authentication}login/legacy`,
          form,
          { withCredentials: true },
        );
      }),
      map(
        (res) => {
          return this.setSession(res);
        },
      ),
    );
  }

  logout(): Observable<boolean> {
    return this._http.get<boolean>(`${APP_CONFIG.authentication}logout`, { withCredentials: true });
  }

  getUserInfo(): Observable<IUserInfo> {
    return this._http.get<IUserInfo>(`${APP_CONFIG.authentication}user`, { withCredentials: true });
  }

  getPublicKey(): Observable<any> {
    return this._http.get<any>(`${APP_CONFIG.middlewareLegacyAuth}publicKey`).pipe(
      map(
        (res) => {
          return res.publicKey;
        },
      ),
    );
  }

  /**
   * Takes un unencrypted string as params and returns it encrypted with the public key provided by the backend service
   */
  encrypt(unencrypted: string, publicKey: string): string {
    if (!publicKey) {
      throw new Error('Can\'t encrypt without public key');
    } else {
      const key = new NodeRSA(publicKey);
      const encrypted = key.encrypt(unencrypted, 'base64');
      return encrypted;
    }
  }
}
